####
# WARNING: DO NOT EDIT: this code is automatically generated, see overlay_generator/generate_listeners_module.py
####

from enum import Enum


class NOTIFICATIONS(Enum):
	# InvitationNotificationService
	INVITATION_RECEIVED = 0
	INVITATION_SENT = 1
	INVITATION_DELETED = 2
	INVITATION_UPDATED = 3

	# ContactNotificationService
	CONTACT_NEW = 4
	CONTACT_DELETED = 5
	CONTACT_DETAILS_UPDATED = 6
	CONTACT_PHOTO_UPDATED = 7

	# GroupNotificationService
	GROUP_NEW = 8
	GROUP_DELETED = 9
	GROUP_NAME_UPDATED = 10
	GROUP_PHOTO_UPDATED = 11
	GROUP_DESCRIPTION_UPDATED = 12
	GROUP_PENDING_MEMBER_ADDED = 13
	GROUP_PENDING_MEMBER_REMOVED = 14
	GROUP_MEMBER_JOINED = 15
	GROUP_MEMBER_LEFT = 16
	GROUP_OWN_PERMISSIONS_UPDATED = 17
	GROUP_MEMBER_PERMISSIONS_UPDATED = 18

	# DiscussionNotificationService
	DISCUSSION_NEW = 19
	DISCUSSION_LOCKED = 20
	DISCUSSION_TITLE_UPDATED = 21
	DISCUSSION_SETTINGS_UPDATED = 22

	# MessageNotificationService
	MESSAGE_RECEIVED = 23
	MESSAGE_SENT = 24
	MESSAGE_DELETED = 25
	MESSAGE_BODY_UPDATED = 26
	MESSAGE_UPLOADED = 27
	MESSAGE_DELIVERED = 28
	MESSAGE_READ = 29
	MESSAGE_LOCATION_RECEIVED = 30
	MESSAGE_LOCATION_SENT = 31
	MESSAGE_LOCATION_SHARING_START = 32
	MESSAGE_LOCATION_SHARING_UPDATE = 33
	MESSAGE_LOCATION_SHARING_END = 34
	MESSAGE_REACTION_ADDED = 35
	MESSAGE_REACTION_UPDATED = 36
	MESSAGE_REACTION_REMOVED = 37

	# AttachmentNotificationService
	ATTACHMENT_RECEIVED = 38
	ATTACHMENT_UPLOADED = 39

	# CallNotificationService
	CALL_INCOMING_CALL = 40
	CALL_RINGING = 41
	CALL_ACCEPTED = 42
	CALL_DECLINED = 43
	CALL_BUSY = 44
	CALL_ENDED = 45
		



import asyncio
import logging
import uuid

from olvid import datatypes, OlvidAdminClient
from ClientHolder import ClientHolder
from utils.tools_group import create_standard_group


async def test_backup(client_holder: ClientHolder, fast_mode: bool):
	test_random_nonce: str = str(uuid.uuid4())
	try:
		if fast_mode:
			logging.warn(f"Skip backup tests in fast mode")
			return

		####
		# feed with data before backup
		####
		# create a group with everyone
		logging.info(f"Creating group")
		await create_standard_group(client_holder, client_holder.clients[0], test_random_nonce, "")

		# create admin client key to backup, with storage
		admin_client_key: datatypes.ClientKey = await client_holder.admin_client.admin_client_key_new(test_random_nonce, 0)
		admin_storage_element: datatypes.StorageElement = datatypes.StorageElement(key=test_random_nonce, value="admin-storage-to-backup")
		await OlvidAdminClient(identity_id=0, client_key=admin_client_key.key).storage_set(admin_storage_element.key, admin_storage_element.value)

		# feed client storage
		for c in client_holder.clients:
			await c.storage_set(test_random_nonce, f"to-backup-{c.identity.id}")
			async for d in c.discussion_list():
				await c.discussion_storage_set(discussion_id=d.id, key=test_random_nonce, value=f"to-backup-{c.identity.id}")

		####
		# create backup
		####
		await client_holder.admin_client.admin_backup_now()

		####
		# then get backup and check it contains enough data
		####
		backup_key: str = await client_holder.admin_client.admin_backup_key_get()
		backup = await client_holder.admin_client.admin_backup_get(backup_key=backup_key)

		# check admin backup
		if backup.admin_backup.admin_client_key_count == 0 or backup.admin_backup.admin_client_key_count == 0:
			logging.warn(f"Skip backup tests: nothing in admin backup")
			return

		# check profile backups
		profile_to_backup: list[datatypes.Backup.ProfileBackup] = []
		for c in client_holder.clients:
			valid_profiles = [profile for profile in backup.profile_backups if profile.profile_display_name == c.identity.display_name]
			if len(valid_profiles) != 1:
				raise Exception(f"cannot find profile for {c.identity.display_name} in backup")
			profile_to_backup.append(valid_profiles[0])
			snapshot: datatypes.Backup.ProfileBackup.Snapshot = sorted(profile_to_backup[0].snapshots, key=lambda s: s.timestamp)[-1]
			if snapshot.client_key_count == 0:
				logging.error(f"{c.identity.display_name}: snapshot does not contain client key")
				return
			if snapshot.contact_count == 0:
				logging.error(f"{c.identity.display_name}: snapshot does not contain contacts")
				return
			if snapshot.storage_elements_count == 0:
				logging.error(f"{c.identity.display_name}: snapshot does not storage elements")
				return
			if snapshot.group_count == 0:
				logging.warn(f"{c.identity.display_name}: snapshot doest not contain groups ({snapshot})")

		####
		# delete data
		####
		# delete test identities
		for c in client_holder.clients:
			await client_holder.admin_client.admin_identity_delete(c.identity.id)
			logging.info(f"Deleted identity: {c.identity.id}")

		# delete admin client key
		await client_holder.admin_client.admin_client_key_delete(client_key=admin_client_key.key)

		# wait for cascade deletion to complete
		logging.info(f"Wait for deletion to be effective")
		await asyncio.sleep(5)

		####
		# restore admin client key and storage
		####
		await client_holder.admin_client.admin_backup_restore_admin_backup(backup_key=backup_key)

		# check admin client key is back
		new_admin_client_key = await client_holder.admin_client.admin_client_key_get(client_key=admin_client_key.key)
		assert admin_client_key == new_admin_client_key, f"{admin_client_key} != {new_admin_client_key}"

		# check admin storage
		new_admin_storage_element_value: str = await OlvidAdminClient(identity_id=0, client_key=admin_client_key.key).storage_get(test_random_nonce)
		assert admin_storage_element.value == new_admin_storage_element_value, f"{admin_storage_element.value} != {new_admin_storage_element_value}"

		####
		# restore profile
		####
		for c in client_holder.clients:
			profile = [profile for profile in backup.profile_backups if profile.profile_display_name == c.identity.display_name][0]
			snapshot: datatypes.Backup.ProfileBackup.Snapshot = sorted(profile.snapshots, key=lambda s: s.timestamp)[-1]

			logging.debug(f"{c.identity.display_name}: snapshot to restore\n{snapshot}")

			response = await client_holder.admin_client.admin_backup_restore_profile_snapshot(backup_key=backup_key, id=snapshot.id, new_device_name=test_random_nonce)

			assert response.restored_identity.display_name.__eq__(profile.profile_display_name)
			assert c.client_key in [ck.key for ck in response.restored_client_keys], f"{c.client_key} not in {response.restored_client_keys}"
			# check contact
			assert len([c async for c in c.contact_list()]) == snapshot.contact_count, "Invalid contact count"
			# check groups
			assert len([c async for c in c.group_list()]) == snapshot.group_count, "Invalid group count"
			assert len([g async for g in c.group_list(filter=datatypes.GroupFilter(name_search=test_random_nonce))]) == 1, "Do not found group"
			# check storage
			assert (await c.storage_get(test_random_nonce)).startswith("to-backup"), "cannot find global storage"
			# check discussion storage
			async for d in c.discussion_list():
				assert (await c.discussion_storage_get(discussion_id=d.id, key=test_random_nonce)).startswith("to-backup"), "cannot find discussion storage"

	# try to clean created elements
	finally:
		# remove admin client key
		try:
			await client_holder.admin_client.admin_client_key_delete(test_random_nonce)
		except:
			pass

		# clean client storage
		for c in client_holder.clients:
			try:
				await c.storage_unset(test_random_nonce)
			except:
				pass
			try:
				async for d in c.discussion_list():
					await c.discussion_storage_unset(discussion_id=d.id, key=test_random_nonce)
			except:
				pass



import asyncclick as click

from .interactive_main import interactive_main
from .interactive_tree import interactive_tree
from .tools.ClientSingleton import ClientSingleton
from .tools.click_wrappers import WrapperGroup


@click.group(invoke_without_command=True, cls=WrapperGroup)
@click.option("-i", "--identity", "identity", default=-1, type=click.INT, help="Id of the identity you want to use")
@click.option("-e", "--foreach", "for_each", is_flag=True, help="Run this command for each identity (ignored if -i is set)")
@click.option("-k", "--key", "client_key", default="", type=click.STRING, help="Impersonate a client key on start")
@click.option("-v", "--version", "version", is_flag=True, help="Show program version and exit")
@click.argument("arguments", required=False, nargs=-1, type=click.UNPROCESSED)
async def root_tree(arguments: list[str], identity: int = 0, client_key: str = "", for_each: bool = False, version: bool = False):
	if version:
		from olvid import __version__
		print(__version__)
		return
	# global options to interactive and non-interactive
	if identity != -1:
		ClientSingleton.set_current_identity_id(identity_id=identity)
	if client_key:
		key = await ClientSingleton.get_client().admin_client_key_get(client_key=client_key)
		await ClientSingleton.impersonate_client_key(key)

	# start interactive mode
	if not arguments:
		await interactive_main()
		return

	# enable script mode
	ClientSingleton.enable_script_mode()

	# non interactive mode
	try:
		# add root tree commands to this tree
		interactive_tree.commands.update(root_tree.commands)

		# set identity (or run for_each)
		identity_ids: list[int] = [i.id async for i in ClientSingleton.get_client().admin_identity_list()] if for_each and identity == -1 else [identity]
		for identity_id in identity_ids:
			if not client_key:
				ClientSingleton.set_current_identity_id(identity_id)

			# parse command
			async with await interactive_tree.make_context(info_name="olvid-cli", args=list(arguments)) as ctx:
				# run command
				await interactive_tree.invoke(ctx)
	except click.UsageError as e:
		# NoArgsIsHelpError error is raised for incomplete commands (`message`, `settings discussion`)
		# in that case e.format_message already returns help message
		if type(e).__name__ == "NoArgsIsHelpError":
			print(e.format_message())
		# for other error show error message in red and help message in white
		else:
			click.echo(click.style(e.format_message(), fg="red"))
			if e.ctx is not None:
				print(e.ctx.get_help())
	# raised in some conditions for invalid commands (for example when you call a command group: `cli identity`)
	except click.exceptions.Exit:
		pass
	except Exception as e:
		click.echo(click.style(f"Unexpected exception: {e}", fg="red"))
		raise e



# for __version__ use python version syntax: https://packaging.python.org/en/latest/discussions/versioning/
# for alpha version: set x.x.xa0
# for post version: set x.x.x.post1
__version__ = "2.0.0a3"
# for __docker_version__: use same version as daemon (x.x.x, x.x.x-alpha)
__docker_version__ = "2.0.0-alpha"

if __name__ == "__main__":
	print(__version__, end="")



import os
import grpc
from typing import Optional

class GrpcTlsConfiguration:
	"""
	If daemon uses TLS you will need to set it up. You can create and pass a GrpcTlsConfiguration as a parameter,
	use environment or add symlinks to appropriate files to have a more persistent configuration.
	There are two valid TLS configuration: a "simple" configuration (use daemon certificate to encrypt communications)
	and a mutual authentication configuration where you need a signed certificate to authenticate on daemon.

	To set up a simple configuration, choose between one of this:
	- use OlvidClient tls_configuration __init__ parameter and pass an OlvidClient.GrpcSimpleTlsConfiguration instance,
	 	with daemon certificate file path as parameter
	- set OLVID_SERVER_CERTIFICATE_PATH in environment to the daemon certificate file path.
	- create a file / a symlink named .server.pem to the daemon certificate file.

	To set up a mutual authentication configuration, choose between one of this:
	- use OlvidClient tls_configuration __init__ parameter and pass an OlvidClient.GrpcMutualAuthTlsConfiguration instance.
		Set root_certificate_path to the CA certificate file path, certificate_chain_path to your certificate file path,
		and private_key_path to your private key file path.
	- use environment variables:
		- OLVID_ROOT_CERTIFICATE_PATH: CA certificate file path
		- OLVID_CERTIFICATE_CHAIN_PATH: your client certificate file path
		- OLVID_PRIVATE_KEY_PATH: your client key file path
	- create a file / a symlink to get a persistent configuration:
		- .ca.pem: CA certificate file path
		- .client.pem: your client certificate file path
		- .client.key: your client key file path
	"""
	# create grpc channel credentials from current instance
	def get_channel_credentials(self) -> grpc.ChannelCredentials:
		raise NotImplementedError()

	# try to load a tls configuration using env and default file names
	@staticmethod
	def load_implicit_configuration() -> Optional:
		raise NotImplementedError()


# Use to connect to a daemon using https
class GrpcSslConfiguration(GrpcTlsConfiguration):
	# create grpc channel credentials from current instance
	def get_channel_credentials(self) -> grpc.ChannelCredentials:
		return grpc.ssl_channel_credentials()

	# try to load a tls configuration using env and default file names
	@staticmethod
	def load_implicit_configuration() -> Optional:
		return


class GrpcSimpleTlsConfiguration(GrpcTlsConfiguration):
	_SERVER_CERTIFICATE_PATH_VARIABLE_NAME: str = "OLVID_SERVER_CERTIFICATE_PATH"
	_SERVER_CERTIFICATE_DEFAULT_PATH: str = ".server.pem"

	def __init__(self, server_certificate_path: str):
		self.server_certificate_path = server_certificate_path

	def get_channel_credentials(self) -> grpc.ChannelCredentials:
		with open(self.server_certificate_path, "rb") as fd:
			server_certificate: bytes = fd.read()
		return grpc.ssl_channel_credentials(root_certificates=server_certificate)

	@staticmethod
	def load_implicit_configuration() -> Optional[GrpcTlsConfiguration]:
		if os.getenv(GrpcSimpleTlsConfiguration._SERVER_CERTIFICATE_PATH_VARIABLE_NAME):
			return GrpcSimpleTlsConfiguration(os.getenv(GrpcSimpleTlsConfiguration._SERVER_CERTIFICATE_PATH_VARIABLE_NAME))
		elif os.path.isfile(GrpcSimpleTlsConfiguration._SERVER_CERTIFICATE_DEFAULT_PATH):
			return GrpcSimpleTlsConfiguration(GrpcSimpleTlsConfiguration._SERVER_CERTIFICATE_DEFAULT_PATH)
		else:
			return None


class GrpcMutualAuthTlsConfiguration(GrpcTlsConfiguration):
	_ROOT_CERTIFICATE_PATH_VARIABLE_NAME: str = "OLVID_ROOT_CERTIFICATE_PATH"
	_ROOT_CERTIFICATE_DEFAULT_PATH: str = ".ca.pem"

	_CERTIFICATE_CHAIN_PATH_VARIABLE_NAME: str = "OLVID_CERTIFICATE_CHAIN_PATH"
	_CERTIFICATE_CHAIN_DEFAULT_PATH: str = ".client.pem"

	_PRIVATE_KEY_PATH_VARIABLE_NAME: str = "OLVID_PRIVATE_KEY_PATH"
	_PRIVATE_KEY_DEFAULT_PATH: str = ".client.key"

	def __init__(self, root_certificate_path: str, certificate_chain_path: str, private_key_path: str):
		self.root_certificate_path = root_certificate_path
		self.certificate_chain_path = certificate_chain_path
		self.private_key_path = private_key_path

	def get_channel_credentials(self) -> grpc.ChannelCredentials:
		with open(self.root_certificate_path, "rb") as fd:
			root_certificate: bytes = fd.read()
		with open(self.certificate_chain_path, "rb") as fd:
			certificate_chain: bytes = fd.read()
		with open(self.private_key_path, "rb") as fd:
			private_key: bytes = fd.read()

		return grpc.ssl_channel_credentials(
			root_certificates=root_certificate,
			certificate_chain=certificate_chain,
			private_key=private_key
		)

	@staticmethod
	def load_implicit_configuration() -> Optional[GrpcTlsConfiguration]:
		if os.getenv(GrpcMutualAuthTlsConfiguration._ROOT_CERTIFICATE_PATH_VARIABLE_NAME):
			root_certificate_path = os.getenv(GrpcMutualAuthTlsConfiguration._ROOT_CERTIFICATE_PATH_VARIABLE_NAME)
		elif os.path.isfile(GrpcMutualAuthTlsConfiguration._ROOT_CERTIFICATE_DEFAULT_PATH):
			root_certificate_path = GrpcMutualAuthTlsConfiguration._ROOT_CERTIFICATE_DEFAULT_PATH
		else:
			return None

		if os.getenv(GrpcMutualAuthTlsConfiguration._CERTIFICATE_CHAIN_PATH_VARIABLE_NAME):
			certificate_chain_path = os.getenv(GrpcMutualAuthTlsConfiguration._CERTIFICATE_CHAIN_PATH_VARIABLE_NAME)
		elif os.path.isfile(GrpcMutualAuthTlsConfiguration._CERTIFICATE_CHAIN_DEFAULT_PATH):
			certificate_chain_path = GrpcMutualAuthTlsConfiguration._CERTIFICATE_CHAIN_DEFAULT_PATH
		else:
			return None

		if os.getenv(GrpcMutualAuthTlsConfiguration._PRIVATE_KEY_PATH_VARIABLE_NAME):
			private_key_path = os.getenv(GrpcMutualAuthTlsConfiguration._PRIVATE_KEY_PATH_VARIABLE_NAME)
		elif os.path.isfile(GrpcMutualAuthTlsConfiguration._PRIVATE_KEY_DEFAULT_PATH):
			private_key_path = GrpcMutualAuthTlsConfiguration._PRIVATE_KEY_DEFAULT_PATH
		else:
			return None

		return GrpcMutualAuthTlsConfiguration(
			root_certificate_path=root_certificate_path,
			certificate_chain_path=certificate_chain_path,
			private_key_path=private_key_path
		)



from google.protobuf.json_format import Parse, ParseError

from ..interactive_tree import interactive_tree
from ..tools.cli_tools import *
from ..tools.click_wrappers import WrapperGroup
from ...core import errors


#####
# key
#####
@interactive_tree.group("key", short_help="(admin) manage your identities client keys", cls=WrapperGroup)
def key_tree():
	pass


#####
# key new
#####
@key_tree.command("new", help="create a new client key associated with an identity")
@click.argument("key_name", required=True, type=str)
@click.argument("identity_id", required=True, type=int)
async def key_new(key_name: str, identity_id: int):
	if not ClientSingleton.is_client_admin():
		print_error_message("Cannot manage client keys when impersonating a non admin key")
		return

	client_key = await ClientSingleton.get_client().admin_client_key_new(name=key_name, identity_id=identity_id)
	print_normal_message(f"Client key created: {client_key.key}", client_key.key)


#####
# key rm
#####
@key_tree.command("rm", help="delete a client key")
@click.argument("client_keys", nargs=-1, required=True, type=click.STRING)
async def key_delete(client_keys: tuple[str]):
	if not ClientSingleton.is_client_admin():
		print_error_message("Cannot manage client keys when impersonating a non admin key")
		return

	for key in client_keys:
		await ClientSingleton.get_client().admin_client_key_delete(key)
		print_command_result(f"Key deleted: {key}")


#####
# key get
#####
# noinspection PyProtectedMember
@key_tree.command("get", help="list existing client keys")
@click.option("-a", "--all", "get_all", is_flag=True)
@click.argument("keys", nargs=-1, type=click.STRING)
@click.option("-f", "--fields", "fields", type=str)
@click.option("--filter", "filter_", type=str)
async def key_get(get_all: bool, keys: tuple[str], fields: str, filter_: str = ""):
	if not ClientSingleton.is_client_admin():
		print_error_message("Cannot manage client keys when impersonating a non admin key")
		return

	# build filter
	key_filter: datatypes.ClientKeyFilter = datatypes.ClientKeyFilter()
	if filter_:
		try:
			parsed_message = Parse(filter_, datatypes.ClientKeyFilter()._to_native(key_filter))
			key_filter = datatypes.ClientKeyFilter._from_native(parsed_message)
		except ParseError as e:
			print_error_message(f"Cannot parse filter: {e}")
			return

	if get_all or not keys:
		async for client_key in ClientSingleton.get_client().admin_client_key_list(filter=key_filter):
			filter_fields_and_print_normal_message(client_key, fields)
	else:
		for client_key in list(keys):
			client_key = await ClientSingleton.get_client().admin_client_key_get(client_key)
			filter_fields_and_print_normal_message(client_key, fields)


#####
# identity impersonate
#####
@key_tree.command("impersonate", help="connect the cli to daemon using a specific client key (necessary to access storage api)")
@click.argument("client_key", required=False, type=click.STRING, default="")
async def key_impersonate(client_key: str):
	if client_key:
		# if not currently on admin client swap to default admin client to create a new client
		if not ClientSingleton.is_client_admin():
			await ClientSingleton.use_default_client()
		try:
			key = await ClientSingleton.get_client().admin_client_key_get(client_key=client_key)
		except errors.NotFoundError:
			print_error_message("Client key not found")
			return
		await ClientSingleton.impersonate_client_key(client_key=key)
		print_normal_message(f"Impersonating: {key.name}: {key.identity_id}", f"{key.identity_id}")
	# swap to default admin client
	else:
		if ClientSingleton.is_default_client():
			print_normal_message("Already using default key", "")
		else:
			print_normal_message("Back to default key", "")
			await ClientSingleton.use_default_client()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: olvid/daemon/datatypes/v1/discussion.proto
# Protobuf Python Version: 6.33.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    2,
    '',
    'olvid/daemon/datatypes/v1/discussion.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*olvid/daemon/datatypes/v1/discussion.proto\x12\x19olvid.daemon.datatypes.v1\"~\n\nDiscussion\x12\x0e\n\x02id\x18\x01 \x01(\x04R\x02id\x12\x14\n\x05title\x18\x02 \x01(\tR\x05title\x12\x1f\n\ncontact_id\x18\x03 \x01(\x04H\x00R\tcontactId\x12\x1b\n\x08group_id\x18\x04 \x01(\x04H\x00R\x07groupIdB\x0c\n\nidentifier\"\xa7\x02\n\x10\x44iscussionFilter\x12I\n\x04type\x18\x01 \x01(\x0e\x32\x30.olvid.daemon.datatypes.v1.DiscussionFilter.TypeH\x01R\x04type\x88\x01\x01\x12\x1f\n\ncontact_id\x18\x02 \x01(\x04H\x00R\tcontactId\x12\x1b\n\x08group_id\x18\x03 \x01(\x04H\x00R\x07groupId\x12&\n\x0ctitle_search\x18\x04 \x01(\tH\x02R\x0btitleSearch\x88\x01\x01\":\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x0c\n\x08TYPE_OTO\x10\x01\x12\x0e\n\nTYPE_GROUP\x10\x02\x42\x0c\n\nidentifierB\x07\n\x05_typeB\x0f\n\r_title_searchB\xc8\x01\n\x1d\x63om.olvid.daemon.datatypes.v1B\x0f\x44iscussionProtoP\x01Z\x0folvid.io/daemon\xa2\x02\x03ODD\xaa\x02\x19Olvid.Daemon.Datatypes.V1\xca\x02\x19Olvid\\Daemon\\Datatypes\\V1\xe2\x02%Olvid\\Daemon\\Datatypes\\V1\\GPBMetadata\xea\x02\x1cOlvid::Daemon::Datatypes::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'olvid.daemon.datatypes.v1.discussion_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\035com.olvid.daemon.datatypes.v1B\017DiscussionProtoP\001Z\017olvid.io/daemon\242\002\003ODD\252\002\031Olvid.Daemon.Datatypes.V1\312\002\031Olvid\\Daemon\\Datatypes\\V1\342\002%Olvid\\Daemon\\Datatypes\\V1\\GPBMetadata\352\002\034Olvid::Daemon::Datatypes::V1'
  _globals['_DISCUSSION']._serialized_start=73
  _globals['_DISCUSSION']._serialized_end=199
  _globals['_DISCUSSIONFILTER']._serialized_start=202
  _globals['_DISCUSSIONFILTER']._serialized_end=497
  _globals['_DISCUSSIONFILTER_TYPE']._serialized_start=399
  _globals['_DISCUSSIONFILTER_TYPE']._serialized_end=457
# @@protoc_insertion_point(module_scope)



import os.path
from google.protobuf.json_format import Parse, ParseError

from ..interactive_tree import interactive_tree

from ..tools.cli_tools import *
from ..tools.click_wrappers import WrapperGroup


#####
# message
#####
@interactive_tree.group("message", help="send and manage messages", cls=WrapperGroup)
def message_tree():
	pass


#####
# message get
#####
# noinspection PyProtectedMember
@message_tree.command("get", help="list current identity messages")
@click.option("-d", "--discussion", type=int, help="get all messages in given discussion id")
@click.option("-u", "--unread", "unread", is_flag=True, help="get only unread messages")
@click.argument("message_ids", required=False, nargs=-1, type=click.STRING)
@click.option("--filter", "filter_", type=str)
@click.option("-f", "--fields", "fields", type=str)
async def message_get(message_ids: tuple[str], fields: str, discussion: int = 0, filter_: str = "", unread: bool = False):
	# build filter
	message_filter: datatypes.MessageFilter = datatypes.MessageFilter()
	if filter_:
		try:
			parsed_message = Parse(filter_, datatypes.MessageFilter()._to_native(message_filter))
			message_filter = datatypes.MessageFilter._from_native(parsed_message)
		except ParseError as e:
			print_error_message(f"Cannot parse filter: {e}")
			return

	# all messages for identity
	if message_ids:
		for message_id in message_ids:
			filter_fields_and_print_normal_message(await ClientSingleton.get_client().message_get(string_to_message_id(message_id)), fields)
	elif discussion:
		message_filter.discussion_id = discussion
		async for message in ClientSingleton.get_client().message_list(filter=message_filter, unread=unread):
			filter_fields_and_print_normal_message(message, fields)
	else:
		async for message in ClientSingleton.get_client().message_list(filter=message_filter, unread=unread):
			filter_fields_and_print_normal_message(message, fields)


#####
# message rm
#####
# noinspection PyProtectedMember
@message_tree.command("rm", help="delete messages")
@click.option("-d", "--discussion", "discussion_id", type=int, help="delete all messages in given discussion id")
@click.option("-a", "--all", "all_opt", type=bool, is_flag=True, help="delete all messages for given identity id")
@click.option("-e", "--everywhere", type=bool, is_flag=True, help="Delete messages everywhere")
@click.option("--filter", "filter_", type=str)
@click.argument("message_ids", nargs=-1, type=click.STRING)
async def message_delete(message_ids: tuple[str], discussion_id: int, all_opt: bool, everywhere: bool, filter_: str = ""):
	# build filter
	message_filter: datatypes.MessageFilter = datatypes.MessageFilter()
	if filter_:
		try:
			parsed_message = Parse(filter_, datatypes.MessageFilter()._to_native(message_filter))
			message_filter = datatypes.MessageFilter._from_native(parsed_message)
		except ParseError as e:
			print_error_message(f"Cannot parse filter: {e}")
			return

	if (message_ids):
		for message_id in message_ids:
			await ClientSingleton.get_client().message_delete(message_id=string_to_message_id(message_id), delete_everywhere=everywhere)
			print_command_result(f"Message deleted: {message_id}")
	elif discussion_id and everywhere:
		print_error_message(f"Cannot delete a discussion everywhere")
	elif discussion_id:
		await ClientSingleton.get_client().discussion_empty(discussion_id=discussion_id)
		print_command_result(f"Emptied discussion: {discussion_id}")
	elif (all_opt):
		async for message in ClientSingleton.get_client().message_list(filter=message_filter):
			await ClientSingleton.get_client().message_delete(message_id=message.id, delete_everywhere=everywhere)
			print_command_result(f"Message deleted: {message.id}")
	else:
		raise click.exceptions.UsageError("Specify messages to delete ")


#####
# message send
#####
@message_tree.command("send", help="send text message in a given discussion")
@click.option("-c", "--contact", "use_contact", is_flag=True, help="Use passed id as a contact id")
@click.option("-g", "--group", "use_group", is_flag=True, help="Use passed id as a group id")
@click.option("-r", "--reply", "reply_id", type=click.STRING, default=None)
@click.option("-o", "--once", "read_once", is_flag=True)
@click.option("-e", "--existence", "existence_duration", type=click.INT, default=0)
@click.option("-v", "--visibility", "visibility_duration", type=click.INT, default=0)
@click.option("-a", "--attachment", "attachment_paths", type=str, default=(), multiple=True)
@click.argument("discussion_id", nargs=1, type=click.INT)
@click.argument("body", nargs=-1, type=click.STRING, required=False)
async def message_send(discussion_id: int, use_contact: bool, use_group: bool, body: tuple[str], attachment_paths: tuple[str], reply_id: str, read_once: bool, existence_duration: int, visibility_duration: int):
	# check options consistency
	if use_contact and use_group:
		print_error_message("You cannot use --contact and --group simultaneously")
		print_normal_message(click.get_current_context().get_help(), "")
		return
	if not body and not attachment_paths:
		print_error_message("You must specify at least a body or an attachment path")
		print_normal_message(click.get_current_context().get_help(), "")
		return

	# check attachment existence
	for path in attachment_paths:
		if not os.path.isfile(path):
			print_error_message(f"File not found: {path}")
			return
		if not os.access(path, os.R_OK):
			print_error_message(f"Cannot read file: {path}")
			return

	ephemerality = datatypes.MessageEphemerality(
		read_once=read_once,
		visibility_duration=visibility_duration,
		existence_duration=existence_duration
	)
	if use_contact:
		discussion_id = (await ClientSingleton.get_client().discussion_get_by_contact(contact_id=discussion_id)).id
	elif use_group:
		discussion_id = (await ClientSingleton.get_client().discussion_get_by_group(group_id=discussion_id)).id

	message, attachments = await ClientSingleton.get_client().message_send_with_attachments_files(
		discussion_id=discussion_id,
		body=" ".join(body),
		reply_id=string_to_message_id(reply_id) if reply_id is not None else None,
		ephemerality=ephemerality,
		file_paths=list(attachment_paths)
	)
	print_normal_message(message, message.id)
	for attachment in attachments:
		print_normal_message(attachment, attachment.id)


#####
# message update
#####
@message_tree.command("update", help="Update a message text body")
@click.argument("message_id", nargs=1, type=click.STRING)
@click.argument("new_body", nargs=-1, type=click.STRING, required=True)
async def message_send(message_id: str, new_body: tuple[str]):
	new_body = " ".join(new_body)
	await ClientSingleton.get_client().message_update_body(string_to_message_id(message_id), new_body)
	print_command_result("Message updated")


#####
# message react
#####
@message_tree.command("react", help="react to a given message")
@click.argument("message_id", nargs=1, type=click.STRING)
@click.argument("reaction", nargs=1, required=False, type=click.STRING)
async def message_send_reaction(message_id: str, reaction: str):
	await ClientSingleton.get_client().message_react(string_to_message_id(message_id), reaction)
	print_command_result("Reaction sent")


#####
# message refresh
#####
@message_tree.command("refresh", help="fore downloading last messages from server")
async def message_update():
	await ClientSingleton.get_client().message_refresh()
	print_command_result("Refreshed messages")

#####
# message location
#####
@message_tree.group(name="location")
def message_location_tree():
	pass

@message_location_tree.command("send", help="send a location message")
@click.option("-p", "--preview", "preview_file", type=click.Path(exists=True, readable=True, file_okay=True, dir_okay=False, resolve_path=True))
@click.option("--address", type=click.STRING)
@click.option("-a", "--altitude", type=click.FLOAT)
@click.option("-p", "--precision", type=click.FLOAT)
@click.argument("discussion_id", nargs=1, type=click.INT)
@click.argument("latitude", nargs=1, type=click.FLOAT)
@click.argument("longitude", nargs=1, type=click.FLOAT)
async def message_send_location(discussion_id: int, latitude: float, longitude: float, address: str = "", preview_file: str = None, altitude: float = None, precision: float = None):
	if preview_file is not None:
		with open(preview_file, "rb") as fd:
			preview_payload: bytes = fd.read()
		message: datatypes.Message = await ClientSingleton.get_client().message_send_location(
			discussion_id=discussion_id, latitude=latitude, longitude=longitude,
			address=address, preview_filename=preview_file, preview_payload=preview_payload, altitude=altitude, precision=precision)
	else:
		message: datatypes.Message = await ClientSingleton.get_client().message_send_location(
			discussion_id=discussion_id, latitude=latitude, longitude=longitude, address=address, altitude=altitude, precision=precision)
	print_normal_message(message, message.id)

@message_location_tree.command("start", help="start to share a location")
@click.option("-a", "--altitude", type=click.FLOAT)
@click.option("-p", "--precision", type=click.FLOAT)
@click.argument("discussion_id", nargs=1, type=click.INT)
@click.argument("latitude", nargs=1, type=click.FLOAT)
@click.argument("longitude", nargs=1, type=click.FLOAT)
async def message_location_start(discussion_id: int, latitude: float, longitude: float, altitude: float = None, precision: float = None):
	message = await ClientSingleton.get_client().message_start_location_sharing(discussion_id=discussion_id, latitude=latitude, longitude=longitude, altitude=altitude, precision=precision)
	print_normal_message(message, message.id)

@message_location_tree.command("update", help="update a location sharing message")
@click.option("-a", "--altitude", type=click.FLOAT)
@click.option("-p", "--precision", type=click.FLOAT)
@click.argument("message_id", nargs=1, type=click.STRING)
@click.argument("latitude", nargs=1, type=click.FLOAT)
@click.argument("longitude", nargs=1, type=click.FLOAT)
async def message_location_update(message_id: str, latitude: float, longitude: float, altitude: float = None, precision: float = None):
	await ClientSingleton.get_client().message_update_location_sharing(message_id=string_to_message_id(message_id), latitude=latitude, longitude=longitude, altitude=altitude, precision=precision)

@message_location_tree.command("end", help="end a location sharing message")
@click.argument("message_id", nargs=1, type=click.STRING)
async def message_location_end(message_id: str):
	await ClientSingleton.get_client().message_end_location_sharing(message_id=string_to_message_id(message_id))
